//  ███████╗██████╗  █████╗ ██╗    ██╗███╗   ██╗
//  ██╔════╝██╔══██╗██╔══██╗██║    ██║████╗  ██║
//  ███████╗██████╔╝███████║██║ █╗ ██║██╔██╗ ██║
//  ╚════██║██╔═══╝ ██╔══██║██║███╗██║██║╚██╗██║
//  ███████║██║     ██║  ██║╚███╔███╔╝██║ ╚████║
//  ╚══════╝╚═╝     ╚═╝  ╚═╝ ╚══╝╚══╝ ╚═╝  ╚═══╝
//
//   ██████╗ ██████╗ ███╗   ██╗███╗   ██╗███████╗ ██████╗████████╗██╗ ██████╗ ███╗   ██╗
//  ██╔════╝██╔═══██╗████╗  ██║████╗  ██║██╔════╝██╔════╝╚══██╔══╝██║██╔═══██╗████╗  ██║
//  ██║     ██║   ██║██╔██╗ ██║██╔██╗ ██║█████╗  ██║        ██║   ██║██║   ██║██╔██╗ ██║
//  ██║     ██║   ██║██║╚██╗██║██║╚██╗██║██╔══╝  ██║        ██║   ██║██║   ██║██║╚██╗██║
//  ╚██████╗╚██████╔╝██║ ╚████║██║ ╚████║███████╗╚██████╗   ██║   ██║╚██████╔╝██║ ╚████║
//   ╚═════╝ ╚═════╝ ╚═╝  ╚═══╝╚═╝  ╚═══╝╚══════╝ ╚═════╝   ╚═╝   ╚═╝ ╚═════╝ ╚═╝  ╚═══╝
//
// Instantiate a new connection from the connection manager.

const OrientDB = require('../../../private/machinepack-orient');

module.exports = function spawnConnection(datastore) {
  return new Promise((resolve, reject) => {
    // Validate datastore
    if (!datastore || !datastore.manager || !datastore.config) {
      reject(new Error('Spawn Connection requires a valid datastore.'));
    }

    OrientDB.getConnection({
      manager: datastore.manager,
      meta: datastore.config,
    }).switch({
      error: function error(err) {
        reject(err);
      },
      failed: function failedToConnect(err) {
        reject(err);
      },
      success: async function success({ connection }) {
        // Create a session from a pool of sessions.
        try {
          const session = await connection.pool.acquire();
          resolve(session);
        } catch (error) {
          resolve(error);
        }
      },
    });
  });
};
